@@ -72,9 +72,10 @@ value for ``image`` is 0.
 
 ::
 
-  layer: mylayer.bst
+  layer:
+  - mylayer.bst
 
-``layer`` is a build dependency which provides the top
+``layer`` is a list of build dependencies which provides the upper
 layer. Integration commands are not run. So you may want to use depend
 on ``compose`` element to run those. `layer` is optional. If not
 provided, parent layers will be just used and new configuration will
@@ -158,7 +159,7 @@ OCI specific:
 ::
 
   Labels: {"a": "b"}
-  StopSignals: "SIGKILL"
+  StopSignal: "SIGKILL"
 
 Docker specific:
 
@@ -167,7 +168,7 @@ Docker specific:
   Memory: 2048
   MemorySwap: 4096
   CpuShares: 2
-  Heathcheck:
+  Healthcheck:
     Test: ["CMD", "/bin/test", "param"]
     Interval: 50000000000
     Timeout: 10000000000
@@ -296,6 +297,8 @@ def safe_path(path):
         return norm
 
 class OciElement(Element):
+    BST_ARTIFACT_VERSION = 1
+
     def configure(self, node):
         self.node_validate(node, [
             'mode', 'gzip',
@@ -383,7 +386,7 @@ class OciElement(Element):
                     'CpuShares', 'Healthcheck',
                 ]
                 oci_config = [
-                    'Labels', 'StopSignals'
+                    'Labels', 'StopSignal'
                 ]
 
                 self.node_validate(config, common_config + (docker_config if self.mode == 'docker' else oci_config))
@@ -399,8 +402,8 @@ class OciElement(Element):
                         config_value[member] = \
                             int(self.node_subst_member(config, member))
 
-                for member in ['ExposedPort', 'Volumes',
-                               'Env', 'EntryPoint', 'Cmd']:
+                for member in ['ExposedPorts', 'Volumes',
+                               'Env', 'Entrypoint', 'Cmd']:
                     if member in config:
                         config_value[member] = \
                             self.node_subst_list(config, member)
@@ -408,7 +411,7 @@ class OciElement(Element):
                 if 'Labels' in config:
                     labels = self.node_get_member(config, Mapping, 'Labels')
                     config_value['Labels'] = {}
-                    for k, v in self.node_items(config, labels):
+                    for k, v in self.node_items(labels):
                         config_value['Labels'][k] = v
 
                 if 'Healthcheck' in config:
@@ -417,19 +420,19 @@ class OciElement(Element):
                         'Test', 'Interval',
                         'Timeout', 'Retries'
                     ])
-                    config['Healthcheck'] = {}
+                    config_value['Healthcheck'] = {}
                     if 'Test' in healthcheck:
-                        config['Healthcheck']['Test'] = self.node_subst_list(healthcheck, 'Test')
+                        config_value['Healthcheck']['Test'] = self.node_subst_list(healthcheck, 'Test')
                     for member in ['Interval', 'Timeout', 'Retries']:
                         if member in healthcheck:
-                            config['Healthcheck'][member] = int(self.node_subst_member(healthcheck, member))
+                            config_value['Healthcheck'][member] = int(self.node_subst_member(healthcheck, member))
 
                 image_value['config'] = config_value
             if 'annotations' in image:
                 image_value['annotations'] = {}
                 annotations = \
                     self.node_get_member(image, Mapping, 'annotations')
-                for k, _ in self.node_items(config, annotations):
+                for k, _ in self.node_items(annotations):
                     v = self.node_subst_member(annotations, k)
                     image_value['annotations'][k] = v
 
@@ -465,6 +468,7 @@ class OciElement(Element):
         legacy_parent = None
 
         config = {}
+        config['created'] = '2011-11-11T11:11:11Z'
         if 'author' in image:
             config['author'] = image['author']
         config['architecture'] = image['architecture']
