@@ -199,7 +199,7 @@ class DockerRegistryV2Client():
     # Returns:
     #    (str, str): A tuple of the manifest content as text, and its content hash
     def manifest(self, image_path, reference,
-                 architecture=default_architecture(), os=default_os()):
+                 architecture, _os):
         accept_types = ['application/vnd.docker.distribution.manifest.v2+json',
                         'application/vnd.docker.distribution.manifest.list.v2+json']
 
@@ -239,12 +239,11 @@ class DockerRegistryV2Client():
             for sub in manifest['manifests']:
                 if sub['platform']['architecture'] == architecture and sub['platform']['os']:
                     sub_digest = sub['digest']
-                    return self.manifest(image_path, sub_digest, architecture=architecture, os=os)
-                else:
-                    raise DockerManifestError(
-                        "No images found for architecture {}, OS {}".format(architecture, os),
-                        manifest=response.text)
-        elif manifest['mediaType'] == 'application/vnd.docker.distribution.manifest.v2+json':
+                    return self.manifest(image_path, sub_digest, architecture=architecture, _os=_os)
+            raise DockerManifestError(
+                "No images found for architecture {}, OS {}".format(architecture, _os),
+                manifest=response.text)
+        if manifest['mediaType'] == 'application/vnd.docker.distribution.manifest.v2+json':
             return response.text, our_digest
         else:
             raise DockerManifestError(
@@ -274,6 +273,12 @@ class DockerRegistryV2Client():
 
 class DockerSource(Source):
 
+    BST_FORMAT_VERSION = 1
+
+    # url is deprecated, but accept it as a valid key so that we can raise
+    # a nicer warning.
+    CONFIG_KEYS = ['architecture', 'registry-url', 'image', 'os', 'ref', 'track', 'url'] + Source.COMMON_CONFIG_KEYS
+
     # Docker identifies images by a content digest calculated from the image's
     # manifest. This corresponds well with the concept of a 'ref' in
     # BuildStream. However, Docker theoretically supports multiple hash
@@ -290,9 +295,7 @@ class DockerSource(Source):
         return 'sha256:' + ref
 
     def configure(self, node):
-        # url is deprecated, but accept it as a valid key so that we can raise
-        # a nicer warning.
-        self.node_validate(node, ['registry-url', 'image', 'ref', 'track', 'url'] + Source.COMMON_CONFIG_KEYS)
+        self.node_validate(node, DockerSource.CONFIG_KEYS)
 
         if 'url' in node:
             raise SourceError("{}: 'url' parameter is now deprecated, "
@@ -339,7 +342,7 @@ class DockerSource(Source):
         with self.timed_activity("Fetching image manifest for image: '{}:{}' from: {}"
                                  .format(self.image, self.tag, self.registry_url)):
             try:
-                manifest, digest = self.client.manifest(self.image, self.tag)
+                manifest, digest = self.client.manifest(self.image, self.tag, self.architecture, self.os)
             except DockerManifestError as e:
                 self.log("Problem downloading manifest", detail=e.manifest)
                 raise
@@ -381,7 +384,7 @@ class DockerSource(Source):
                 manifest = self._load_manifest()
             except FileNotFoundError:
                 try:
-                    manifest_text, digest = self.client.manifest(self.image, self.digest)
+                    manifest_text, digest = self.client.manifest(self.image, self.digest, self.architecture, self.os)
                 except requests.RequestException as e:
                     raise SourceError(e) from e
 
