@@ -39,6 +39,8 @@
 #
 # Authors:
 #        Thomas Coldrick <thomas.coldrick@codethink.co.uk>
+#        William Salmon <will.salmon@codethink.co.uk>
+#        Richard Dale <richard.dale@codethink.co.uk>
 
 
 """
@@ -98,6 +100,9 @@ git-tag - extension of BuildStream git plugin to track latest tag
    # If not set, this will default to 'True'
    checkout-submodules: True
 
+   # Optionally specify whether or not to clone Git LFS objects
+   use-lfs: False
+
    # If your repository has submodules, explicitly specifying the
    # url from which they are to be fetched allows you to easily
    # rebuild the same sources from a different location. This is
@@ -117,21 +122,31 @@ git-tag - extension of BuildStream git plugin to track latest tag
    # Fetch a full clone instead of a shallow clone.
    full-clone: False
 
+**Configurable Warnings:**
+
+This plugin provides the following :ref:`configurable warnings <configurable_warnings>`:
+
+- ``git:unused-lfs`` - There are files with Git LFS attributes but 'use-lfs' is not explicitly set.
+
 """
 
 import os
 import errno
 import re
-from collections import Mapping
+from collections.abc import Mapping
 from io import StringIO
 
 from configparser import RawConfigParser
 
 from buildstream import Source, SourceError, Consistency, SourceFetcher
 from buildstream import utils
+from urllib.parse import urljoin
 
 GIT_MODULES = '.gitmodules'
+GIT_ATTRIBUTES = '.gitattributes'
 
+# Warnings
+WARN_UNUSED_GITLFS = "unused-lfs"
 
 # Because of handling of submodules, we maintain a GitMirror
 # for the primary git source and also for each submodule it
@@ -299,7 +314,7 @@ class GitTagMirror(SourceFetcher):
                 fail="Failed to retrieve list of remotes in {}".format(mirror),
                 cwd=mirror
             )
-            if remote_name not in remotes:
+            if remote_name not in remotes.splitlines():
                 self.source.call(
                     [self.source.host_git, 'remote', 'add', remote_name, url],
                     fail="Failed to add remote {} with url {}".format(remote_name, url),
@@ -339,6 +354,24 @@ class GitTagMirror(SourceFetcher):
 
         # Check if the ref is really there
         rc = self.source.call([self.source.host_git, 'cat-file', '-t', self.ref], cwd=mirror)
+        if not rc == 0:
+            return False
+
+        if self.source.use_lfs is not None:
+            return True
+        ## This tries to test if git-lfs is used in this repo and if it is then mandates that you have set
+        ## whether or not to use git-lfs with the use-lfs option.
+        showrc, attrs = self.source.check_output(
+                [self.source.host_git, 'show', "{}:{}".format(self.ref, GIT_ATTRIBUTES)],
+                cwd=mirror)
+        if showrc == 0 and ('filter=lfs' in attrs or 'diff=lfs' in attrs or 'merge=lfs' in attrs):
+            self.source.warn(
+                    "{}: Git LFS not configured but LFS objects exist"
+                    .format(self.source),
+                    detail="a filter, diff or merge is set to use lfs in {}"
+                    .format(GIT_ATTRIBUTES),
+                    warning_token=WARN_UNUSED_GITLFS
+            )
         return rc == 0
 
     def assert_ref(self):
@@ -354,10 +387,8 @@ class GitTagMirror(SourceFetcher):
 
             if exit_code == 128:
                 self.source.info("Unable to find tag for specified branch name '{}'".format(tracking))
-                _, output = self.source.check_output(
-                        [self.source.host_git, 'rev-parse', tracking],
-                        fail="Unable to find commit for specified branch name '{}'".format(tracking),
-                        cwd=self.mirror)
+                return None, None
+
             tracking = output.rstrip('\n')
 
         else:
@@ -397,13 +428,22 @@ class GitTagMirror(SourceFetcher):
                          fail="Failed to create git mirror {} in directory: {}".format(mirror, fullpath),
                          fail_temporarily=True)
 
+        if self.source.use_lfs:
+            self.set_origin_url(directory)
+        
+        ## This turns git-lfs off if it is installed but we dont want to use it.
+        my_env = os.environ.copy()
+        if not self.source.use_lfs:
+            my_env['GIT_LFS_SKIP_SMUDGE'] = 'TRUE'
         self.source.call([self.source.host_git, 'checkout', '--force', self.ref],
                          fail="Failed to checkout git ref {}".format(self.ref),
+                         env=my_env,
                          cwd=fullpath)
 
     def init_workspace(self, directory):
         fullpath = os.path.join(directory, self.path)
-        url = self.source.translate_url(self.url)
+        url = self.source.translate_url(self.url,
+                                        primary=self.primary)
 
         self.source.call([self.source.host_git, 'clone', '--no-checkout', self.mirror, fullpath],
                          fail="Failed to clone git mirror {} in directory: {}".format(self.mirror, fullpath),
@@ -413,8 +453,13 @@ class GitTagMirror(SourceFetcher):
                          fail='Failed to add remote origin "{}"'.format(url),
                          cwd=fullpath)
 
+        ## This turns git-lfs off if it is installed but we dont want to use it.
+        my_env = os.environ.copy()
+        if not self.source.use_lfs:
+            my_env['GIT_LFS_SKIP_SMUDGE'] = 'TRUE'
         self.source.call([self.source.host_git, 'checkout', '--force', self.ref],
                          fail="Failed to checkout git ref {}".format(self.ref),
+                         env=my_env,
                          cwd=fullpath)
 
     # List the submodules (path/url tuples) present at the given ref of this repo
@@ -486,58 +531,49 @@ class GitTagMirror(SourceFetcher):
             return None
 
 
-class GitTagSource(Source):
+    # Git-lfs can not retrieve objects from a file based directory, it needs something like github or gitlab
+    # to serve them. This function can be used when gitlfs is requested to set the checkout repo to point to
+    # the original url rather than the mirror when it is about to do the full checkout.
+    # This happens after the --no-checkout clone so that as little is pulled from the remote as posible.
+    def set_origin_url(self, directory):
+        fullpath = os.path.join(directory, self.path)
+        _, origin_url = self.source.check_output([self.source.host_git, 'config', '--get', 'remote.origin.url'],
+                         fail='Failed to get origin url "{}"'.format(self.mirror),
+                         cwd=self.mirror)
+        self.source.call([self.source.host_git, 'config', 'remote.origin.url', origin_url],
+                         fail="Failed to set origin url {}".format(origin_url),
+                         cwd=fullpath)
+
+class AbstractGitTagSource(Source):
     # pylint: disable=attribute-defined-outside-init
 
-    BST_FORMAT_VERSION = 1
-
     def configure(self, node):
-        ref = self.node_get_member(node, str, 'ref', '') or None
+        config_keys = ['url', 'ref', 'full-clone', 'use-lfs']
+        config_keys.extend(self.get_extra_config_keys())
+        config_keys.extend(Source.COMMON_CONFIG_KEYS)
 
-        config_keys = ['url', 'track', 'track-tags', 'track-extra', 'ref', 'submodules', 'checkout-submodules', 'match', 'exclude', 'full-clone']
-        self.node_validate(node, config_keys + Source.COMMON_CONFIG_KEYS)
+        self.node_validate(node, config_keys)
 
         self.original_url = self.node_get_member(node, str, 'url')
         self.full_clone = self.node_get_member(node, bool, 'full-clone', False)
-        self.mirror = GitTagMirror(self, '', self.original_url, ref, primary=True, full_clone=self.full_clone)
-        self.tracking = self.node_get_member(node, str, 'track', None)
-        self.track_extra = self.node_get_member(node, list, 'track-extra', default=[])
-        self.track_tags = self.node_get_member(node, bool, 'track-tags', False)
-        self.match = self.node_get_member(node, list, 'match', [])
-        self.exclude = self.node_get_member(node, list, 'exclude', [])
+        self.use_lfs = self.node_get_member(node, bool, 'use-lfs', None)
 
-        # At this point we now know if the source has a ref and/or a track.
-        # If it is missing both then we will be unable to track or build.
-        if self.mirror.ref is None and self.tracking is None:
-            raise SourceError("{}: Git sources require a ref and/or track".format(self),
-                              reason="missing-track-and-ref")
-
-        self.checkout_submodules = self.node_get_member(node, bool, 'checkout-submodules', True)
-        self.submodules = []
-
-        # Parse a dict of submodule overrides, stored in the submodule_overrides
-        # and submodule_checkout_overrides dictionaries.
-        self.submodule_overrides = {}
-        self.submodule_checkout_overrides = {}
-        modules = self.node_get_member(node, Mapping, 'submodules', {})
-        for path, _ in self.node_items(modules):
-            submodule = self.node_get_member(modules, Mapping, path)
-            url = self.node_get_member(submodule, str, 'url', '') or None
-
-            # Make sure to mark all URLs that are specified in the configuration
-            if url:
-                self.mark_download_url(url, primary=False)
-
-            self.submodule_overrides[path] = url
-            if 'checkout' in submodule:
-                checkout = self.node_get_member(submodule, bool, 'checkout')
-                self.submodule_checkout_overrides[path] = checkout
+        self.extra_configure(node)
 
         self.mark_download_url(self.original_url)
 
+    def extra_configure(self, node):
+        pass
+
+    def get_extra_config_keys(self):
+        return []
+
     def preflight(self):
         # Check if git is installed, get the binary at the same time
         self.host_git = utils.get_host_tool('git')
+        if self.use_lfs:
+            self.call([self.host_git, 'lfs', '--version'],
+                              fail="Git lfs not installed")
 
     def get_unique_key(self):
         # Here we want to encode the local name of the repository and
@@ -545,21 +581,16 @@ class GitTagSource(Source):
         # from another location, it should not effect the cache key.
         key = [self.original_url, self.mirror.ref]
 
-        # Only modify the cache key with checkout_submodules if it's something
-        # other than the default behaviour.
-        if self.checkout_submodules is False:
-            key.append({"checkout_submodules": self.checkout_submodules})
+        key.extend(self.get_extra_unique_key())
 
-        # We want the cache key to change if the source was
-        # configured differently, and submodules count.
-        if self.submodule_overrides:
-            key.append(self.submodule_overrides)
-
-        if self.submodule_checkout_overrides:
-            key.append({"submodule_checkout_overrides": self.submodule_checkout_overrides})
+        if self.use_lfs:
+            key.append("use-lfs")
 
         return key
 
+    def get_extra_unique_key(self):
+        return []
+
     def get_consistency(self):
         if self.have_all_refs():
             return Consistency.CACHED
@@ -567,6 +598,25 @@ class GitTagSource(Source):
             return Consistency.RESOLVED
         return Consistency.INCONSISTENT
 
+    def init_workspace(self, directory):
+        # XXX: may wish to refactor this as some code dupe with stage()
+        self.mirror.ensure_trackable()
+
+        with self.timed_activity('Setting up workspace "{}"'.format(directory), silent_nested=True):
+            self.mirror.init_workspace(directory)
+            self.extra_init_workspace(directory)
+
+    def extra_init_workspace(self, directory):
+        pass
+
+    def stage(self, directory):
+        with self.timed_activity("Staging {}".format(self.mirror.url), silent_nested=True):
+            self.mirror.stage(directory)
+            self.extra_stage(directory)
+
+    def extra_stage(self, directory):
+        pass
+
     def load_ref(self, node):
         ref = self.node_get_member(node, str, 'ref', None)
         self.mirror.set_ref(ref)
@@ -605,23 +655,89 @@ class GitTagSource(Source):
                        branch, track_tags=self.track_tags, track_args=track_args)
                        for branch in branches])
 
+            candidates.pop(None, None)
+
             # Find latest candidate ref from all branches
             # Update self.mirror.ref, node.ref
-            ret = max(candidates, key=candidates.get)
+            ret = max(candidates, key=candidates.get, default=None)
 
         return ret
 
-    def init_workspace(self, directory):
-        # XXX: may wish to refactor this as some code dupe with stage()
-        self.mirror.ensure_trackable()
+    def have_all_refs(self):
+        if not self.mirror.has_ref():
+            return False
+
+        return self.have_all_extra_refs()
+
+    def have_all_extra_refs(self):
+        return True
+
+
+class GitTagSource(AbstractGitTagSource):
+    # pylint: disable=attribute-defined-outside-init
+
+    BST_FORMAT_VERSION = 2
+
+    def get_extra_config_keys(self):
+        return ['track', 'track-extra', 'track-tags', 'match', 'exclude', 'checkout-submodules', 'submodules']
+
+    def extra_configure(self, node):
+        ref = self.node_get_member(node, str, 'ref', '') or None
+
+        self.tracking = self.node_get_member(node, str, 'track', None)
+        self.track_extra = self.node_get_member(node, list, 'track-extra', default=[])
+        self.track_tags = self.node_get_member(node, bool, 'track-tags', False)
+        self.match = self.node_get_member(node, list, 'match', [])
+        self.exclude = self.node_get_member(node, list, 'exclude', [])
+        self.checkout_submodules = self.node_get_member(node, bool, 'checkout-submodules', True)
+        self.submodules = []
+
+        # Parse a dict of submodule overrides, stored in the submodule_overrides
+        # and submodule_checkout_overrides dictionaries.
+        self.submodule_overrides = {}
+        self.submodule_checkout_overrides = {}
+        modules = self.node_get_member(node, Mapping, 'submodules', {})
+        for path, _ in self.node_items(modules):
+            submodule = self.node_get_member(modules, Mapping, path)
+            url = self.node_get_member(submodule, str, 'url', '') or None
+
+            # Make sure to mark all URLs that are specified in the configuration
+            if url:
+                self.mark_download_url(url, primary=False)
+
+            self.submodule_overrides[path] = url
+            if 'checkout' in submodule:
+                checkout = self.node_get_member(submodule, bool, 'checkout')
+                self.submodule_checkout_overrides[path] = checkout
+
+        self.mirror = GitTagMirror(self, '', self.original_url, ref, primary=True, full_clone=self.full_clone)
+
+        # At this point we now know if the source has a ref and/or a track.
+        # If it is missing both then we will be unable to track or build.
+        if self.mirror.ref is None and self.tracking is None:
+            raise SourceError("{}: Git sources require a ref and/or track".format(self),
+                              reason="missing-track-and-ref")
+
+    def get_extra_unique_key(self):
+        key = []
+
+        # We want the cache key to change if the source was
+        # configured differently, and submodules count.
+        if self.submodule_overrides:
+            key.append(self.submodule_overrides)
+
+        if self.submodule_checkout_overrides:
+            key.append({"submodule_checkout_overrides": self.submodule_checkout_overrides})
+
+        return key
+
+    def extra_init_workspace(self, directory):
         self.refresh_submodules()
 
-        with self.timed_activity('Setting up workspace "{}"'.format(directory), silent_nested=True):
-            self.mirror.init_workspace(directory)
-            for mirror in self.submodules:
-                mirror.init_workspace(directory)
+        for mirror in self.submodules:
+            mirror.init_workspace(directory)
 
-    def stage(self, directory):
+    def extra_stage(self, directory):
 
         # Need to refresh submodule list here again, because
         # it's possible that we did not load in the main process
@@ -632,10 +748,8 @@ class GitTagSource(Source):
 
         # Stage the main repo in the specified directory
         #
-        with self.timed_activity("Staging {}".format(self.mirror.url), silent_nested=True):
-            self.mirror.stage(directory)
-            for mirror in self.submodules:
-                mirror.stage(directory)
+        for mirror in self.submodules:
+            mirror.stage(directory)
 
     def get_source_fetchers(self):
         yield self.mirror
@@ -646,10 +760,7 @@ class GitTagSource(Source):
     ###########################################################
     #                     Local Functions                     #
     ###########################################################
-    def have_all_refs(self):
-        if not self.mirror.has_ref():
-            return False
-
+    def have_all_extra_refs(self):
         self.refresh_submodules()
         for mirror in self.submodules:
             if not os.path.exists(mirror.mirror) and not os.path.exists(mirror.fetch_mirror):
@@ -685,6 +796,8 @@ class GitTagSource(Source):
 
             ref = self.mirror.submodule_ref(path)
             if ref is not None:
+                if url.startswith("./") or url.startswith("../"):
+                    url = urljoin("{}/".format(self.mirror.url), url)
                 mirror = GitTagMirror(self, path, url, ref)
                 submodules.append(mirror)
 
