@@ -295,7 +295,8 @@ class Cli():
         with open(os.devnull) as devnull:
             sys.stdin = devnull
 
-            capture = MultiCapture(out=True, err=True, in_=False, Capture=FDCapture)
+            capture = MultiCapture(in_=None, out=FDCapture(1), err=FDCapture(2))
+
             capture.start_capturing()
 
             try:
